;****************************************************************************
;
; Name: 	ASPI1DSK.INC
;
; Description:  Segment declarations, device driver definitions, etc., for
;		ASPI1DSK.SYS.  Any DOS technical reference manual can give
;		a more complete definition of the device driver structures.
;
;****************************************************************************

;****************************************************************************
;		  %%% SEGMENT/GROUP DECLARATIONS %%%
;****************************************************************************

CDDHeader	segment word public 'CODE'	;Character device driver
CDDHeader	ends				;  header segment

ScsiCode        segment word public 'CODE'      ;Code segment to keep after
ScsiCode        ends				;  driver INIT

ScsiData        segment word public 'CODE'      ;Data segment to keep after
ScsiData        ends				;  driver INIT

ThrowAwaySeg	segment word public 'CODE'	;Segments after this one are
ThrowAwaySeg	ends				;  thrown away after INIT

DataInit	segment word public 'CODE'	;Data segment to throw away
DataInit	ends				;  after driver INIT
						
CodeInit	segment word public 'CODE'      ;Code segment to throw away
CodeInit	ends			        ;  after driver INIT

;; Lump both data and code to one segment

ASPISeg  	group	CDDHeader,ScsiCode,ScsiData,ThrowAwaySeg,DataInit,CodeInit

DataOffset	equ	offset ASPISeg:  	;offset equates
CodeOffset	equ	offset ASPISeg:  	;  ...

;****************************************************************************
;		    %%% MISCELLANEOUS EQUATES %%%
;****************************************************************************

VER_HI			equ	'1'	;Driver Revision 1.0
VER_LO			equ	'0'

LF			equ     10	;Linefeed
CR			equ     13	;Carriage Return

DOS_TIME_SEG		equ	40h	;Memory address of current time
DOS_TIME_OFF		equ	6Ch
DOS_TIME_ROLL		equ	0B0h	;Timer Roll over
TIME_OUT		equ	10*18	;Ten second time out

;****************************************************************************
;		      %%% DOS REQUEST HEADER %%%
;****************************************************************************
REQ_HDR			struc
			db	?		;Length
	unit		db	?		;Unit number
	command		db	?		;Command code
	status		dw	?		;Status return
			db	8 dup (?)	;Reserved bytes
REQ_HDR			ends

;status valuse for the STATUS field

STAT_DONE		equ	01h		;Function complete status
STAT_BUSY		equ	0200h		;Busy bit for Removable Media call
STAT_WTPRT		equ	00H
STAT_DNRDY		equ	02h		;Drive not ready
STAT_CMDERR		equ	03h		;Invalid command code error
STAT_CRC		equ	04h		;CRC error
STAT_SEEK		equ	06h		;Seek error
STAT_UNMEDIA		equ	07h		;Unknown media
STAT_SNF		equ	08h		;Sector not found
STAT_GENF		equ	0Ch		;General failure
STAT_ERR		equ	80h		;Error code

;****************************************************************************
;		%%% INITIALIZATION COMMAND HEADER %%%
;****************************************************************************
INIT_HDR		struc
			db	(type REQ_HDR) dup (?)
	units		db	?		;Number of unit
	end_off		dw	?		;Offset of ending address
	end_seg		dw	?		;Segment of ending address
	bpb_off		dw	?		;Offset of BPB array address
	bpb_seg		dw	?		;Segment of BPB array address
	drv_num		db	?		;Driver number
INIT_HDR		ends

;****************************************************************************
;		  %%% Media Check Command Header %%%
;****************************************************************************
MED_HDR			struc
			db	(type REQ_HDR) dup (?)
	media		db 	?		;Media descriptor
	change		db	?		;Change status
MED_HDR			ends


;****************************************************************************
;		   %%% BUILD BPB COMMAND HEADER %%%
;****************************************************************************
BPB_HDR			struc
			db	(type REQ_HDR) dup (?)
			db	?		;Media descriptor
			dw	?		;Offset of transfer addr
			dw	?		;Segment of transfer addr
	bpbptro		dw	?		;Offset of pointer to BPB table
	bpbptrs		dw	?		;Segment of pointer to BPB table 
BPB_HDR 		ends


;****************************************************************************
;		  %%% READ/WRITE COMMAND HEADER %%%
;****************************************************************************
RW_HDR			struc
			db	(type REQ_HDR) dup (?)
			db	?		;Media descriptor
	xfer_off	dw	?		;Transfer addr offset
	xfer_seg	dw	?		;Transfer addr. segment
	count		dw	?		;Sector count (for disk device driver)
	ss_num		dw	?		;Starting sector number
			dd	?
	exsct_nlo	dw	?
	exsct_nhi	dw 	?
RW_HDR			ends

;****************************************************************************
;			%%% BPB STRUCTURE %%%
;****************************************************************************
BPB_STRUC		struc
	byte_sect	dw	? 		;Bytes per sector
	sect_clust	db	?		;Sectors per cluster
	rserv_sect 	dw	?		;Reserved sectors starting at logical sector 0
	num_fat		db	?		;Number of FATs
	num_root	dw	?		;Maximum number of root directory entries
	num_sect	dw	?		;Total number of sectors
	med_descpt	db	?		;Media descriptor
	sect_fat	dw	?		;Number of sectors occupied by a FAT
	sct_ptrk	dw	?		;Sector per track
	num_heads	dw	?		;Number of heads
	hidden_sct	dd	?		;Hidden sector
	ext_numsct  	dd	?	   	;Extended number of sector
BPB_STRUC		ends

;****************************************************************************
;	%%% CUSTOM BPB STRUCTURE - DRIVER APPENDED FIELDS %%%
;****************************************************************************
DRV_BPB			struc
			db	(SIZE BPB_STRUC) dup (?)
	tarlun_num    	db	?		;Target/lun number
	start_secmsb 	db	?		;Start partition sector msb
	start_secmid 	db	?		;Start partition sector mid
	start_seclsb	db	?		;Start partition sector lsb
DRV_BPB			ends

