;****************************************************************************
;
; Name: 	ASPIDEFS.INC
;
; Description:	ASPI definitions (Assembly Language)
;		Refer to the ASPI specification for more information.
;
; NOTES:  All multi-byte elements are intended to be stored in Intel byte
;	  ordering (ie. 80x86).  Some 32 bit fields have been split into
;	  WORD fields to simplify 16-bit programming.
;
;****************************************************************************

SENSE_LEN		equ	14	;Default request sense buffer length

;****************************************************************************
;	   %%% INT 21 Function call definitions for AX %%%
;****************************************************************************
ASPI_OPEN		equ	3D00H	;Open the ASPI driver
ASPI_GET_ENTRY		equ	4402H	;Get the entry point
ASPI_CLOSE		equ	3E00H	;Close the ASPI driver

;****************************************************************************
;		   %%% ASPI Command Definitions %%%
;****************************************************************************
HA_INQUIRY		equ	00h	;Host adapter inquiry
GET_DEV_TYPE		equ	01h	;Get device type
EXEC_SCSI_CMD		equ	02h	;Execute SCSI command
ABORT_SRB		equ	03h	;Abort an SRB
RESET_DEV		equ	04h	;SCSI bus device reset
SET_HA_PARMS		equ	05h	;Set host adapter parameters
GET_DRIVE_INFO		equ	06h	;Get disk drive information

;****************************************************************************
;		     %%% SRB Flag Definitions %%%
;****************************************************************************
SRB_POST		equ	01h	;Interrupt POST enable/disable
SRB_LINKED		equ	02h	;Next SRB field valid/invalid
SRB_DIR_IN		equ	08h	;Inbound data transfer
SRB_DIR_OUT		equ	10h	;Outbound data transfer

;****************************************************************************
;			  %%% SRB Status %%%
;****************************************************************************
SRB_PENDING		equ	00h	;SRB being processed
SRB_COMP		equ	01h	;SRB completed without error
SRB_ABORTED		equ	02h	;SRB aborted
SRB_ABORT_FAIL		equ	03h	;Unable to abort SRB
SRB_ERR 		equ	04h	;SRB completed with error

INV_ASPI_CMD		equ	80h	;Invalid ASPI command
INV_HA_ID		equ	81h	;Invalid host adapter number
DEV_NOT_INST		equ	82h	;SCSI device not installed

;****************************************************************************
;		     %%% Host Adapter Status %%%
;****************************************************************************
HASTAT_OK		equ	00h	;Host adapter did not detect an error
HASTAT_SEL_TO		equ	11h	;Selection Timeout
HASTAT_DO_DU		equ	12h	;Data overrun data underrun
HASTAT_BUS_FREE		equ	13h	;Unexpected bus free
HASTAT_PHASE_ERR	equ	14h	;Target bus phase sequence failure

;****************************************************************************
;	       %%% SCSI REQUEST BLOCK (SRB) HEADER %%%
;****************************************************************************
SRB_Header		struc
	SRB_Cmd		db	?	;ASPI command code
	SRB_Status 	db	?	;Return status
	SRB_HaId 	db	?	;Host Adapter number
	SRB_Flags	db	?	;Request Flags
	SRB_Hdr_Rsvd	dd	?	;Reserved
SRB_Header		ends

;****************************************************************************
;	       %%% SRB - HOST ADAPTER INQUIRY (00) %%%
;****************************************************************************
SRB_HA_Inquiry		struc
			db	(SIZE SRB_Header) dup (?)

	HA_Count	db	?	    ;# of Host Adapters
	HA_SCSI_ID	db	?	    ;SCSI ID of the host adapter
	HA_ManagerId	db	16 dup (?)  ;SCSI Manager ID String
	HA_Identifier	db	16 dup (?)  ;Host Adapter ID String
	HA_Unique	db	16 dup (?)  ;Host Adapter Unique parameters
SRB_HA_Inquiry		ends

;****************************************************************************
;		  %%% SRB - GET DEVICE TYPE (01) %%%
;****************************************************************************
GDEVBlock		struc
			db	(SIZE SRB_Header) dup (?)

			db	?	;SRB_Target
			db	?	;SRB_Lun
	TargDeviceType	db	?	;Target's peripheral device type
GDEVBlock		ends

;****************************************************************************
;	       %%% SRB - EXECUTE SCSI COMMAND (02) %%%
;****************************************************************************
SRB_Exec_SCSI_Cmd	struc
			db	(SIZE SRB_Header) dup (?)

	SRB_Target	db	?	;Target Id
	SRB_Lun		db	?	;Logical Unit Number
	SRB_BufLenLo	dw	?	;Data Allocation Length
	SRB_BufLenHi	dw	?	;
	SRB_SenseLen	db	?	;Sense Allocation Length
	SRB_BufOffset	dw	?	;Data Buffer Pointer
	SRB_BufSegment	dw	?	;
	SRB_NextOffset	dw	?	;Next request pointer for linking
	SRB_NextSegment	dw	?	;
	SRB_CDBLen	db	?	;CDB Length
	SRB_HaStat	db	?	;Host Adapter Status
	SRB_TargStat	db	?	;Target Status

	SRB_PostOffset	dw	?	;POST routine Offset
	SRB_PostSegment	dw	?	;POST routine CS

	SRB_Reserved	db	34 dup (?) ;Reserved for SCSI Manager Workspace
SRB_Exec_SCSI_Cmd	ends

SRB_with_CDB6		struc
			db	(SIZE SRB_Exec_SCSI_Cmd) dup (?)
	CDBByte 	db	6 dup (?)
	SenseArea6	db	SENSE_LEN dup (?)
SRB_with_CDB6		ends

SRB_with_CDB10		struc
			db	(SIZE SRB_Exec_SCSI_Cmd) dup (?)
			db	10 dup (?)		;CDBByte
	SenseArea10	db	SENSE_LEN dup (?)
SRB_with_CDB10		ends

;****************************************************************************
;		   %%% SRB - ABORT AN SRB (03) %%%
;****************************************************************************
SRB_Abort		struc
			db	(SIZE SRB_Header) dup (?)

	AbortOffset	dw	?	;Offset of SRB to abort
	AbortSegment	dw	?	;Segment of SRB to abort
SRB_Abort		ends

;****************************************************************************
;		 %%% SRB - BUS DEVICE RESET (04) %%%
;****************************************************************************
;
; Use "SRB_Exec_SCSI_Cmd" definition
;

;****************************************************************************
;	    %%% SRB - SET HOST ADAPTER PARAMETERS (05) %%%
;****************************************************************************
;
; Reserved for setting host adapter parameters
;

;****************************************************************************
;	    %%% SRB - GET DISK DRIVE INFORMATION (06) %%%
;****************************************************************************
SRB_GetDrvInfo		struc
			db	(SIZE SRB_Header) dup (?)

			db	?	;SRB_Target
			db	?	;SRB_Lun
	DriveFlags	db	?	;Drive flags byte
	Int13Drive	db	?	;INT 13h drive number
	HeadXlat	db	?	;Head translation
	SectorXlat	db	?	;Sector translation
	GetDrvReserved	db	10 dup (?)
SRB_GetDrvInfo		ends

