PAGE 60,132
TITLE ASPI sample application program (C) 1992 Adaptec, Inc.

;****************************************************************************
;*									    *
;* Copyright 1992 Adaptec, Inc.,  All Rights Reserved.	 		    *
;*									    *
;* This software contains the valuable trade secrets of Adaptec.  The	    *
;* software is protected under copyright laws as an unpublished work of	    *
;* Adaptec.  Notice is for informational purposes only and does not imply   *
;* publication.  The user of this software may make copies of the software  *
;* for use with parts manufactured by Adaptec or under license from Adaptec *
;* and for no other use.						    *
;*									    *
;****************************************************************************

;****************************************************************************
;*
;*  Program Name: ASPIDEMO.EXE
;*
;*  Version:	  1.0a
;*
;*  Source Code:  ASPIDEMO.ASM     ASPIDEFS.INC    SCSIDEFS.INC
;*		  MAKEFILE
;*
;*  Base Code #: 537803-00
;*
;*  Description:  This program provides examples of how to use the ASPI
;*		  programming interface.  The main program obtains the ASPI
;*		  entry point and calls subroutines for the different ASPI
;*		  functions.
;*
;*  1. Obtains  ASPI Entry Point
;*  2. Executes ASPI Host Adapter Inquiry Command
;*  3. Executes ASPI Get Device Type Command
;*  4. Executes ASPI Execute SCSI I/O Command
;*  5. Executes ASPI Reset SCSI Device Command using Posting
;*
;****************************************************************************

		.286
page
		include aspidefs.inc	;include file for ASPI definitions
page
		include scsidefs.inc	;include file for SCSI definitions
page

		.MODEL SMALL
		.STACK 800h

LF		equ     10		;Linefeed
CR		equ     13		;Carriage Return

;****************************************************************************
;			 %%% DATA SEGMENT %%%
;****************************************************************************
		.DATA
SCSIMgrString	db	"SCSIMGR$"      ;SCSI Mgr String (need to get entry)
		dw	0		;NULL-Terminate String
ASPI_Entry	dd	0		;Entry Address stored by GetASPIEntry
SRB		SRB_with_CDB10 <>	;SCSI Request Block
Data_Buffer	db	50 dup (?)	;Data Buffer for Inquiry command
Flag		db	0		;Flag used by post routine

;****************************************************************************
;		       %%% DISPLAY STRINGS %%%
;****************************************************************************
Str_InquiryFail db  "Error issuing SCSI inquiry command",CR,LF,'$'
Str_NoASPI	db  "Unable to open ASPI Manager!",CR,LF,'$'

Str_ASPI_Stat	db  "  ASPI Status (HEX)................ ?",CR,LF,'$'

Str_HA_Inquiry	db  CR,LF,"ASPI HOST ADAPTER INQUIRY COMMAND",CR,LF,'$'
Str_Num_HA	db  "  Number of Host Adapters.......... ?",CR,LF,'$'
Str_TID_HA	db  "  SCSI ID of Host Adapter #0....... ?",CR,LF,'$'
Str_Mgr_ID	db  "  SCSI Manager ID.................. ????????????????",CR,LF,'$'
Str_HA_ID	db  "  Host Adapter ID.................. ????????????????",CR,LF,'$'

Str_Get_DevTyp	db  CR,LF,"ASPI GET DEVICE TYPE COMMAND",CR,LF,'$'
Str_Dev_Typ	db  "  SCSI Device Type................. ?",CR,LF,'$'

Str_SCSI_IO	db  CR,LF,"ASPI SCSI I/O COMMAND (Inquiry Command)",CR,LF,'$'
Str_PQualTyp	db  "  Qualifier / Device Type (HEX).... ?",CR,LF,'$'
Str_Versions	db  "  ISO / ECMA / ANSI Version (HEX).. ?",CR,LF,'$'
Str_Inq_Dat	db  "  Inquiry Data..................... ",'$'

Str_Dev_Rst1	db  CR,LF,"ASPI RESET DEVICE COMMAND (with Posting)",'$'
Str_CRLF	db  CR,LF,'$'

;****************************************************************************
;			 %%% CODE SEGMENT %%%
;****************************************************************************
		.CODE
start:		mov	ax,@DATA		;initialize ds and es
		mov	ds,ax
		mov	es,ax

;*	1. Obtains ASPI Entry Point
		call	GetASPIEntry		;Get ASPI entry point
		jc	ASPI_Done		;Jump if no manager

;*	2. Executes ASPI Host Adapter Inquiry Command
		lea	si,SRB			;Point to SRB
		call	HAInquiry		;Send ASPI HA Inquiry
		call	Poll_Status		;Wait for status
		call	DispSRBInq		;Display Inquiry Info

;*	3. Executes ASPI Get Device Type Command
		lea	si,SRB			;Point to SRB
		call	GetDevType		;Send ASPI Get Device Type
		call	Poll_Status		;Wait for status
		call	DispDevTyp		;Display Device Type Info

;*	4. Executes ASPI Execute SCSI I/O Command
		lea	si,SRB			;Point to SRB
		call	Load_Inquiry		;Load SCSI Inquiry into SRB
		call	XSCSI_IO		;Execute SCSI I/O
		call	Poll_Status		;Wait for status
		call	DispInquiry		;Display I/O results

;*	5. Executes ASPI Reset SCSI Device Command using Posting
		lea	dx,Str_Dev_Rst1 	;Load the string for outtext
		call	outtext 		;Output the text
		mov	Flag,0			;Reset the post flag
		lea	si,SRB			;Point to SRB
		call	RST_SDevice		;Resets SCSI ID = 2
PostLoop:	sti				;Make sure INTs are enabled
		cmp	Flag,1	 		;Get the post flag
		jnz	PostLoop 		;Loop until SRB is finished
		call	DispStat		;Display status

ASPI_Done:	mov	ax,4C00h
		int	21h			;Return to DOS

;****************************************************************************
; Subroutine Name:  GetASPIEntry
;
; Description:	This gets the ASPI manager's entry point and stores it in
;		'ASPI_Entry.'  If no ASPI manager is found, an appropriate
;		error message is displayed.
;
; Entry:  Nothing
;
; Exit:   The memory location 'ASPI_Entry' is loaded with the entry point
;	  CF set if no ASPI manager found, else reset
;****************************************************************************
GetASPIEntry	proc	near
		mov	ax,ASPI_Open
		lea	dx,SCSIMgrString
		int	21h			;Open ASPI Manager
		jc	NoASPIManager		;Branch if none found
		push	ax			;Save ASPI File Handle

		mov	bx,ax			;BX = File Handle
		mov	ax,ASPI_Get_Entry
		lea	dx,ASPI_Entry		;Store entry point here
		mov	cx,4			;Four bytes to transfer
		int	21h			;Get ASPI entry point

		mov	ax,ASPI_Close
		pop	bx			;BX = ASPI File Handle
		int	21h			;Close ASPI Manager
		clc				;Return with carry clear
		ret

NoASPIManager:	lea	dx,Str_NoASPI		;Display error message, no
		call	outtext 		;  ASPI manager found
		stc				;Return with carry set
		ret
GetASPIEntry	endp

;****************************************************************************
; Subroutine Name:  Poll_Status
;
; Description:	This subroutine polls the ASPI Status Byte and waits for
;		non-zero status.  When status goes non-zero we return.
;		NOTE:  Although this sample code will hange if status never
;		goes non-zero, an actual program should time out!!
;
; Entry:  SI - SCSI Request Block (SRB)
;
; Exit:   SRB has completed
;****************************************************************************
Poll_Status	proc	near
poll_loop:	sti				;Enable interrupts
		mov	al,[si].SRB_Status	;Get the status byte of SRB
		cmp	al,0			;SRB finished?
		jz	poll_loop		;Loop until it is
		ret
Poll_Status	endp

;****************************************************************************
; Subroutine Name:  HAInquiry
;
; Description:	This subroutine sends the Host Adapter Inquiry Command to the
;		ASPI Manager.  First the SRB is totally cleared.  Then the
;		opcode and other fields of the SRB are set for HOST ADAPTER
;		INQUIRY command.
;
; Entry:  SI - SCSI Request Block (SRB)
;
; Exit:   SRB is filled with HAInquiry data
;****************************************************************************
HAInquiry	proc	near
		call	ClearSRB		;Zero out our SRB
		mov	[si].SRB_Cmd,HA_INQUIRY	;ASPI opcode = HA inquiry
		mov	[si].SRB_HaId,0		;Host Adapter 0
		mov	[si].SRB_Flags,0 	;0=no length checking
		mov	[si].SRB_Target,0	;Target 0
		mov	[si].SRB_Lun,0		;LUN 0

						;CALLING ASPI !!
		push	ds			;Push segment of SRB
		push	si			;Push offset of SRB
		call	[ASPI_Entry]		;Call ASPI
		add	sp,4			;Restore stack
		ret
HAInquiry	endp

;****************************************************************************
; Subroutine Name:  GetDevType
;
; Description:	This subroutine sends the Get Device Type Command to the
;		ASPI Manager.  First the SRB is totally cleared then the SRB
;		is loaded with opcode and other parameters for the GET DEVICE
;		TYPE command.
;
; Entry:  SI - SCSI Request Block (SRB)
;
; Exit:   SRB is full of returned data
;****************************************************************************
GetDevType	proc	near
		call	ClearSRB		;Zero out our SRB
		mov	[si].SRB_Cmd,GET_DEV_TYPE ;ASPI opcode = Get Device Type
		mov	[si].SRB_HaId,0		;Host Adapter 0
		mov	[si].SRB_Flags,0 	;0=no length checking
		mov	[si].SRB_Target,0	;Target 0
		mov	[si].SRB_Lun,0		;Lun 0

						;CALLING ASPI !!
		push	ds			;Push segment of SRB
		push	si			;Push offset of SRB
		call	[ASPI_Entry]		;Call ASPI
		add	sp,4			;Restore stack
		ret
GetDevType	endp

;****************************************************************************
; Subroutine Name:  XSCSI_IO
;
; Description:	This subroutine sends the Execute SCSI I/O Command to the
;		ASPI Manager.  It is assumed that the SCSI command has already
;		been loaded into the SRB.  Here the EXEC_SCSI_CMD is loaded
;		into the command code of the SRB along with setting the other
;		fields for a 24 byte read of SCSI INQUIRY data.
;
; Entry:  SI - SCSI Request Block (SRB) - partially initialized
;
; Exit:   SRB is full of returned data
;****************************************************************************
XSCSI_IO	proc	near
		mov	[si].SRB_Cmd,EXEC_SCSI_CMD  ;ASPI opcode = execute SCSI IO
		mov	[si].SRB_HaId,0		;Host Adapter 0
		mov	[si].SRB_Flags,SRB_DIR_IN ;Inbound data transfer
		mov	[si].SRB_Target,0	;Target 0
		mov	[si].SRB_Lun,0		;Lun 0
		mov	[si].SRB_BufLenLo,36	;Buffer xfer length LSB
		mov	[si].SRB_BufLenHi,0	;Buffer xfer length MSB
		mov	[si].SRB_SenseLen,SENSE_LEN
		lea	ax,Data_Buffer
		mov	[si].SRB_BufOffset,ax	;Load Data buffer offset
		mov	[si].SRB_BufSegment,ds	;Load Data buffer segment
		mov	[si].SRB_CDBLen,6	;SCSI CDB Length

						;CALLING ASPI !!
		push	ds			;Push segment of SRB
		push	si			;Push offset of SRB
		call	[ASPI_Entry]		;Call ASPI
		add	sp,4			;Restore stack
		ret
XSCSI_IO	endp

;****************************************************************************
; Subroutine Name:  RST_SDevice
;
; Description:	This subroutine sends the RESET SCSI DEVICE Command to the
;		ASPI Manager.  It attempts to reset SCSI ID 2 instead of
;		ID 0 in case ID 0 is the boot device of your computer.
;		Posting is used by this routine.
;
; Entry:  SI - SCSI Request Block (SRB)
;
; Exit:   Bus device reset in progress
;****************************************************************************
RST_SDevice	proc	near
		call	ClearSRB		;Zero out our SRB
		mov	[si].SRB_Cmd,RESET_DEV	;ASPI opcode = Reset Device
		mov	[si].SRB_HaId,0		;Host Adapter #0
		mov	[si].SRB_Flags,SRB_POST	;Set post bit
		mov	[si].SRB_Target,2	;Target #2
		mov	[si].SRB_Lun,0		;Lun 0
		mov	[si].SRB_BufLenLo,0	;Buffer xfer length LSB
		mov	[si].SRB_BufLenHi,0	;Buffer xfer length MSB
		mov	[si].SRB_SenseLen,SENSE_LEN
		mov	ax,OFFSET ASPI_Post
		mov	[si].SRB_PostOffset,ax	;load post routine offset
		mov	[si].SRB_PostSegment,cs	;load post routine segment

						;CALLING ASPI !!
		push	ds			;push segment of SRB
		push	si			;push offset of SRB
		call	[ASPI_Entry]		;Call ASPI
		add	sp,4			;Restore stack
		ret
RST_SDevice	endp

;****************************************************************************
; Subroutine Name:  Load_Inquiry
;
; Description:	This routine loads the SCSI INQUIRY command into the
;		SRB.
;
; Entry:  SI - SCSI Request Block (SRB)
;
; Exit:   All registers preserved
;****************************************************************************
Load_Inquiry	proc	near
		call	ClearSRB			;Zero out our SRB
		mov	[si].CDBByte._OpCode,SCSI_Inquiry  ;Load opcode
		mov	[si].CDBByte._LUNumber,0	;Set LUN = 0
		mov	[si].CDBByte._LBAMiddl,0	;Set byte 2 = 0
		mov	[si].CDBByte._LBALeast,0	;Set byte 3 = 0
		mov	[si].CDBByte._Length,36 	;Set length = 36
		mov	[si].CDBByte._Control,0 	;Set control = 0
		ret
Load_Inquiry	endp

;****************************************************************************
; Subroutine Name:  ClearSRB
;
; Description:	This routine zeros out the given ASPI SRB.
;
; Entry:  SI - SCSI Request Block (SRB)
;
; Exit:   All registers preserved
;****************************************************************************
ClearSRB	proc	near
		push	es			;Save registers
		push	ax
		push	cx
		push	di

		mov	cx,(SIZE SRB_with_CDB10)/2  ;Zero out the SRB
		mov	ax,ds
		mov	es,ax			;ES=DS
		mov	di,si			;ES:DI points to SRB
		xor	ax,ax			;Store zeros
		cld
		rep	stosw			;Zero out the SRB

		pop	di
		pop	cx
		pop	ax
		pop	es			;Restore registers and
		ret				;return to caller
ClearSRB	endp

;****************************************************************************
; Subroutine Name:  ASPI_Post
;
; Description:	This routine is executed when the Reset Device command
;		completes.  It simply flips a flag so that execution in
;		the RST_SDEV routine can continue.  This flag method is
;		used strictly for example purposes.
;
; Exit:   All registers preserved
;****************************************************************************
ASPI_Post	proc	far
		push	bp
		mov	bp,sp
		push	ds
		push	ax

;;		mov	bx,[bp+6]		;BX = SRB's offset
;;		mov	es,[bp+8]		;ES = SRB's segment

		mov	ax,@DATA		;Point DS to flag's segment
		mov	ds,ax
		mov	Flag,1			;Set done flag

		pop	ax
		pop	ds
		pop	bp
		retf				;return to caller
ASPI_Post	endp

;****************************************************************************
; Subroutine Name:  outtext
;
; Description:	Displays a message on the screen.
;
; Entry:  DS:DX - Pointer to '$' terminated character string
;
; Exit:   Nothing
;****************************************************************************
outtext 	proc	near
		push	ax
		mov	ah,09h			;Do DOS Function Print String
		int	21h			;Call interrupt routine
		pop	ax
		ret
outtext 	endp

;****************************************************************************
; Subroutine Name:  itoa
;
; Description:	Converts the digits of the given value to a '$'-terminated
;		character string.  Appends a carriage return and linefeed
;		to the end of the string as well if requested.
;
; Entry:  AX	- Number to be converted
;	  DS:DX - Pointer to string result
;	  CX	- Base of value (ie. 16 for hex, 10 for dec.)
;
; Exit:   All registers preserved
;****************************************************************************
itoa		proc	near
		pusha
		mov	bx,dx
		or	ax,ax			;Is input value zero?
		jne	NotZero 		;Branch if it's not
		mov	BYTE PTR [bx],'0'       ;  else fill string with
		inc	bx			;  a single '0' and exit
		jmp	SHORT itoa_exit

NotZero:	xor	di,di
NextDigit:	or	ax,ax			;Finished pushing remainders
		je	itoa_pop		;  onto the stack?
		xor	dx,dx
		div	cx			;Let's get the remainder
		push	dx			;Push the remainder
		inc	di			;Increment digit count
		jmp	NextDigit

itoa_pop:	or	di,di			;Finished popping digits?
		je	itoa_exit		;Branch and exit if we are
		pop	dx			;Pop a digit
		cmp	dl,10			;Is it a 0-9 digit?
		jb	DecDigit		;If it is convert to ASCII
		add	dl,'A'-10               ;  else convert to HEX
		jmp	SHORT PlaceChar 	;  alphabet
DecDigit:	add	dl,'0'
PlaceChar:	mov	[bx],dl 		;Store character in string
		inc	bx			;Move to next place in string
		dec	di			;Decrement pop count
		jmp	itoa_pop		;Pop next value

itoa_exit:
		mov	BYTE PTR [bx],CR	;Place Carriage Return &
		mov	BYTE PTR [bx+1],LF	;  and Line Feed and '$'
		mov	BYTE PTR [bx+2],'$'     ;  char at end of string
		jmp	SHORT itoa_exit1

		mov	BYTE PTR [bx],'$'
itoa_exit1:	popa				;Restore all registers
		ret
itoa		endp

;****************************************************************************
; Subroutine Name:  DispSRBInq
;
; Description:	This subroutine displays the information returned in the SRB
;		for the Host Adapter Inquiry Command.
;
; Entry:  SRB - Already filled with Host Adapter Inquiry Data
;
; Exit:   Nothing
;****************************************************************************
DispSRBInq	proc	near
		lea	dx,Str_HA_Inquiry	;load the string for outtext
		call	outtext 		;output the text

		mov	cx,16			;set base= hex for iota
		lea	dx,Str_ASPI_STAT+36	;point to loc in string
		mov	al,SRB_Status		;load value to convert
		mov	ah,0			;zero out ah
		call	itoa			;convert to ASCII in string loc
		lea	dx,Str_ASPI_STAT	;load the string for outtext
		call	outtext 		;output the text

		lea	bx,Str_Num_HA		;Get Number of HA string loc
		mov	al,SRB.HA_Count 	;Get # of ASPI host adapters
		add	al,'0'                  ;AL = ASCII Host Adapter #
		mov	BYTE PTR [bx+36],al	;Place HA # in string
		mov	dx,bx			;load the string for outtext
		call	outtext 		;output the text

		lea	bx,Str_TID_HA		;Get Target ID of HA string loc
		mov	al,SRB.HA_SCSI_ID 	;Get ID
		add	al,'0'                  ;AL = ASCII ID
		mov	BYTE PTR [bx+36],al	;Place HA # in string
		mov	dx,bx			;load the string for outtext
		call	outtext 		;output the text

		mov	cx,16			;ID has 16 bytes (for movsb)
		lea	si,SRB.HA_ManagerId	;load source of string mov
		lea	di,Str_Mgr_ID+36	;load destination of string mov
		rep	movsb			;move string
		lea	dx,Str_Mgr_ID		;Get Manager ID string loc
		call	outtext 		;output the text

		mov	cx,16			;ID has 16 bytes (for movsb)
		lea	si,SRB.HA_Identifier	;load source of string mov
		lea	di,Str_HA_ID+36 	;load destination of string mov
		rep	movsb			;move string
		lea	dx,Str_HA_ID		;Get Manager ID string loc
		call	outtext 		;output the text
		ret
DispSRBInq	endp

;****************************************************************************
; Subroutine Name:  DispDevType
;
; Description:	This subroutine displays the information returned in the SRB
;		for the Get Device Type Command.
;
; Entry:  SRB - Filled with SCSI device type information
;
; Exit:   Nothing
;****************************************************************************
DispDevTyp	proc	near
		lea	dx,Str_Get_DevTyp	;point to Get Dev Type string
		call	outtext 		;output the text

		lea	bx,Str_ASPI_STAT	;point to ASPI Status string
		mov	al,SRB_Status		;Get ASPI status
		add	al,'0'                  ;convert to ASCII
		mov	BYTE PTR [bx+36],al	;place Status in string
		mov	dx,bx			;load the string for outtext
		call	outtext 		;output the text

		lea	bx,Str_Dev_Typ		;Get SCSI Dev Type string loc
		mov	al,SRB.TargDeviceType	;Get # of ASPI host adapters
		add	al,'0'                  ;AL = ASCII Host Adapter #
		mov	BYTE PTR [bx+36],al	;Place HA # in string
		mov	dx,bx			;load the string for outtext
		call	outtext 		;output the text

		ret
DispDevTyp	endp

;****************************************************************************
; Subroutine Name:  DispInquiry
;
; Description:  This routine will display the target's Vendor ID string
;		and Product ID.  This information was received when we
;		did a SCSI Inquiry to the target.
;
; Entry:  Data_Buffer contains device's Inquiry data
;
; Exit:   Nothing
;****************************************************************************
DispInquiry	proc	near
		lea	dx,Str_SCSI_IO		;Load the string for outtext
		call	outtext 		;Output the text

		mov	cx,16			;Set for HEX base (iota call)
		lea	dx,Str_ASPI_STAT[36]	;Point to string location
		mov	al,SRB_Status		;Load value to convert
		mov	ah,0			;Zero out ah
		call	itoa			;Convert to ASCII in string
		lea	dx,Str_ASPI_STAT	;Load the string for outtext
		call	outtext 		;Output the text

		lea	dx,Str_PQualTyp[36]	;Point to string location
		mov	al,Data_Buffer.PerQualTyp ;Load value to convert
		mov	ah,0			;Zero out ah
		call	itoa			;Convert to ASCII in string
		lea	dx,Str_PQualTyp 	;Get Peripheral Qual Dev string
		call	outtext 		;Output the text

		lea	dx,Str_Versions[36]	;Get Peripheral Qual Dev string
		mov	al,Data_Buffer.Versions ;Get Byte from Data Buffer
		mov	ah,0			;Zero out ah
		call	itoa			;Convert to ASCII in string
		lea	dx,Str_Versions 	;Load the string for outtext
		call	outtext 		;Output the text

		lea	dx,Str_Inq_Dat
		call	outtext
		mov	Data_Buffer[32],CR
		mov	Data_Buffer[33],LF
		mov	Data_Buffer[34],'$'
		lea	dx,Data_Buffer[8]	;Display vendor ID and
		call	outtext			;  product ID

		ret
DispInquiry	endp

;****************************************************************************
; Subroutine Name:  DispStat
;
; Description:	This subroutine displays the ASPI status byte
;
;****************************************************************************
DispStat	proc	near
		lea	dx,Str_CRLF
		call	outtext 		;Output the text

		lea	bx,Str_ASPI_STAT	;Point to ASPI Status string
		mov	al,SRB.SRB_Status	;Get ASPI status
		add	al,'0'                  ;Convert to ASCII
		mov	BYTE PTR [bx+36],al	;Place Status in string
		mov	dx,bx			;Load the string for outtext
		call	outtext 		;Output the text

		ret
DispStat	endp

		END	start
